@@ -1,9 +1,9 @@
-package URI;  # $Id: URI.pm,v 1.4 1998/04/09 18:26:57 aas Exp $
+package URI;  # $Id: URI.pm,v 1.7.2.5 1998/09/05 23:44:22 aas Exp $
 
 use strict;
 use vars qw($VERSION $DEFAULT_SCHEME $STRICT $DEBUG);
 
-$VERSION = "0.01";
+$VERSION = "0.03";
 
 $DEFAULT_SCHEME ||= "http";
 #$STRICT = 0;
@@ -17,13 +17,13 @@ my $reserved   = q(;/?:@&=+$,);
 my $mark       = q(-_.!~*'());                                    #'; emacs
 my $unreserved = "A-Za-z0-9\Q$mark\E";
 
-use vars qw($uric $pchar $achar $ppchar);
+use vars qw($uric $pchar $achar $ppchar $scheme_re);
 $uric   = "\Q$reserved\E$unreserved%";
-$pchar  = $uric;  $pchar =~ s,\\[/?;],,g;
-$achar  = $uric;  $achar =~ s,\\[/?],,g;
-$ppchar = $uric;  $ppchar =~ s,\\?,,g;
+$pchar  = $uric;  $pchar  =~ s,\\[/?;],,g;
+$achar  = $uric;  $achar  =~ s,\\[/?],,g;
+$ppchar = $uric;  $ppchar =~ s,\\[?],,g;
 
-my $scheme_re = '[a-zA-Z][a-zA-Z0-9.+\-]*';
+$scheme_re = '[a-zA-Z][a-zA-Z0-9.+\-]*';
 
 #print "$uric\n$achar\n$pchar\n";
 
@@ -49,8 +49,9 @@ sub new
 
 	# We need a scheme to determine which class to use
         my $scheme;
-        $scheme = $1 if $url =~ m/^($scheme_re):/o;
-	unless ($scheme) {
+	if ($url =~ m/^($scheme_re):/so) {
+	    $scheme = $1;
+	} else {
             if (ref $base){
                 $scheme = $base->scheme;
 	    } elsif ($base && $base =~ m/^($scheme_re):/o) {
@@ -65,10 +66,9 @@ sub new
         unless ($impclass) {
             Carp::croak("URI scheme '$scheme' is not supported")
 		if $STRICT;
-            # use generic as fallback
-            require URI::_generic;
-            $impclass = 'URI::_generic';
-            implementor($scheme, $impclass);  # register it
+	    
+	    require URI::_generic;
+	    $impclass = 'URI::_generic';
         }
         # hand-off to scheme specific implementation sub-class
         $self = $impclass->_init($url, $base, $scheme);
@@ -81,32 +81,15 @@ sub _init
 {
     my $class = shift;
     my($str, $base, $scheme) = @_;
-    my $self = bless {}, $class;
-    $self->{'_scheme'} = $scheme;
-    $self->{'_orig_uri'} = $str if $DEBUG;
-    $self->base($base) if $base;
-    $self->_parse($str);
+    $str =~ s/([^$uric\#])/$URI::Escape::escapes{$1}/go;
+    my $self = bless \$str, $class;
     $self;
 }
 
 
-sub _parse
-{
-    my($self, $str) = @_;
-    # <scheme>:<scheme-specific-part>
-    $self->{'scheme'} = $1 if $str =~ s/^($scheme_re)://o;
-    $self->{'fragment'}   = $1 if $str =~ s/\#(.*)//s;
-    $self->{'specific'} = $str;
-}
-
-
 sub implementor
 {
     my($scheme, $impclass) = @_;
-    unless (defined $scheme) {
-        require URI::_generic;
-        return 'URI::_generic';
-    }
     $scheme = lc($scheme);
 
     if ($impclass) {
@@ -128,7 +111,7 @@ sub implementor
     unless (defined @{"${ic}::ISA"}) {
         # Try to load it
         eval "require $ic";
-        die $@ if $@ && $@ !~ /Can\'t locate/;
+        die $@ if $@ && $@ !~ /Can\'t locate.*in \@INC/;
         return unless defined @{"${ic}::ISA"};
     }
 
@@ -149,131 +132,147 @@ sub _init_implementor
 sub clone
 {
     my $self = shift;
-    # this work as long as none of the components are references themselves
-    bless { %$self }, ref $self;
-}
-
-
-sub _elem
-{
-    my $self = shift;
-    my $elem = shift;
-    my $old = $self->{$elem};
-    if (@_) {
-	$self->{$elem} = shift;
-	$self->{'_str'} = '';        # void cached string
-    }
-    $old;
+    my $other = $$self;
+    bless \$other, ref $self;
 }
 
 
 sub scheme
 {
     my $self = shift;
-    my $old = $self->{'scheme'};
-    if (@_) {
-	my $new_scheme = shift;
-	if (defined($new_scheme) && length($new_scheme)) {
-	    # reparse URI with new scheme
-	    my $str = $self->as_string;
-	    $str =~ s/^$scheme_re://o;
-	    my $newself = URI->new("$new_scheme:$str");
-	    %$self = %$newself;  # copy content
-	    bless $self, ref($newself);
-	} else {
-	    $self->{'scheme'} = undef;
-	}
+
+    unless (@_) {
+	return unless $$self =~ /^($scheme_re):/o;
+	return $1;
     }
-    $old;
+
+    my $old;
+    $old = $1 if $$self =~ s/^($scheme_re)://o;
+
+    my $new = shift;
+    if (defined($new) && length($new)) {
+	die "Bad scheme '$new'" unless $new =~ /^$scheme_re$/o;
+	my $newself = URI->new("$new:$$self");
+	$$self = $$newself; 
+	bless $self, ref($newself);
+    } elsif ($$self =~ m/^$scheme_re:/) {
+	warn "Opaque part look like scheme";
+    }
+
+    return $old;
+}
+
+
+sub opaque_part
+{
+    my $self = shift;
+
+    unless (@_) {
+	$$self =~ /^(?:$scheme_re:)?([^\#]*)/o or die;
+	return $1;
+    }
+
+    $$self =~ /^($scheme_re:)?    # optional scheme
+	        ([^\#]*)          # opaque
+                (\#.*)?           # optional fragment
+              $/sx or die;
+
+    my $old_scheme = $1;
+    my $old_opaque = $2;
+    my $old_frag   = $3;
+
+    my $new_opaque = shift;
+    $new_opaque = "" unless defined $new_opaque;
+    $new_opaque =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
+
+    $$self = "";
+    $$self .= $old_scheme if defined $old_scheme;
+    $$self .= $new_opaque;
+    $$self .= $old_frag if defined $old_frag;
+
+    $old_opaque;
 }
 
 
 sub fragment
 {
-    shift->_elem("fragment", @_);
+    my $self = shift;
+    unless (@_) {
+	return unless $$self =~ /\#(.*)/s;
+	return $1;
+    }
+
+    my $old;
+    $old = $1 if $$self =~ s/\#(.*)//s;
+
+    my $new_frag = shift;
+    if (defined $new_frag) {
+	$new_frag =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
+	$$self .= "#$new_frag";
+    }
+    $old;
 }
 
 
 sub as_string
 {
     my $self = shift;
-    if (my $str = $self->{'_str'}) {
-	return $str;  # cached
-    }
-    $self->{'_str'} = $self->_as_string;  # set cache and return
+    $$self;
 }
 
 
-sub _as_string
+sub canonical
 {
     my $self = shift;
-    my $str = "";
-    my($scheme, $specific, $fragment) = @{$self}{qw(scheme specific fragment)};
-    $str = "$scheme:" if $scheme;
-    $specific =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
-    $str .= $specific;
-    if (defined $fragment) {
-	$fragment =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
-	$str .= "#$fragment";
+
+    # Make sure scheme is lowercased
+    my $scheme = $self->scheme;
+    if ($scheme =~ /[A-Z]/) {
+	my $other = $self->clone;
+	$other->scheme(lc $scheme);
+	return $other;
     }
-    $str;
+    # XXX might also want to ensure that we only use either upper or
+    # lower case hex digits in %xx escapes.
+
+    $self;
 }
 
-
-sub base {
-    my $self = shift;
-    my $base  = $self->{'_base'};
-
-    if (@_) { # set
-	my $new_base = shift;
-	$new_base = $new_base->abs if ref($new_base);  # unsure absoluteness
-	$self->{_base} = $new_base;
-    }
-    return unless defined wantarray;
-
-    # The base attribute supports 'lazy' conversion from URI strings
-    # to URI objects. Strings may be stored but when a string is
-    # fetched it will automatically be converted to a URI object.
-    # The main benefit is to make it much cheaper to say:
-    #   URI->new($random_url_string, 'http:')
-    if (defined($base) && !ref($base)) {
-	$self->{'_base'} = $base = URI->new($base);
-    }
-    $base;
-}
-
-
 # Compare two URIs, subclasses will provide a more correct implementation
 sub eq {
     my($self, $other) = @_;
     $other = URI->new($other, $self) unless ref $other;
-    # XXX schemes should be compared case-insensitively
     ref($self) eq ref($other) &&                # same class
-	$self->as_string eq $other->as_string;  # same string
+	$self->canonical->as_string eq $other->canonical->as_string;
 }
 
+1;
+
+__END__
+
 # This is set up as an alias for various methods
 sub _bad_access_method
 {
     my $self = shift;
+    my $type = ref($self) || "URI";
     if ($STRICT) {
-	Carp::croak("Illegal method called for $self->{'_scheme'} URI")
+	Carp::croak("Illegal method called for $type");
     }
     if ($^W && @_) {
-	Carp::carp("Setting not effective for $self->{'_scheme'} URI");
+	Carp::carp("Setting not effective for $type");
     }
     undef;
 }
 
 # generic-URI accessor methods
-*authority = \&_bad_access_method;
-*userinfo  = \&_bad_access_method;
-*host      = \&_bad_access_method;
-*port      = \&_bad_access_method;
-*abs_path  = \&_bad_access_method;
-*path      = \&_bad_access_method;
-*path_segments = \&_bad_access_method;
-*query     = \&_bad_access_method;
+*authority      = \&_bad_access_method;
+*userinfo       = \&_bad_access_method;
+*host           = \&_bad_access_method;
+*port           = \&_bad_access_method;
+*abs_path_query = \&_bad_access_method;
+*path           = \&_bad_access_method;
+*path_segments  = \&_bad_access_method;
+*query          = \&_bad_access_method;
 
 # generic-URI transformation methods
 sub abs { shift->clone; }
